@@ -2,6 +2,8 @@
 # Created by Pearu Peterson, September 2002
 """ Test functions for fftpack.basic module
 """
+from __future__ import division, print_function, absolute_import
+
 __usage__ = """
 Build fftpack:
   python setup_fftpack.py build
@@ -42,14 +44,18 @@ SMALL_PRIME_SIZES = [
 ]
 
 from numpy.random import rand
+
+
 def random(size):
     return rand(*size)
 
+
 def get_mat(n):
     data = arange(n)
     data = add.outer(data,data)
     return data
 
+
 def direct_dft(x):
     x = asarray(x)
     n = len(x)
@@ -59,6 +65,7 @@ def direct_dft(x):
         y[i] = dot(exp(i*w),x)
     return y
 
+
 def direct_idft(x):
     x = asarray(x)
     n = len(x)
@@ -68,48 +75,53 @@ def direct_idft(x):
         y[i] = dot(exp(i*w),x)/n
     return y
 
+
 def direct_dftn(x):
     x = asarray(x)
     for axis in range(len(x.shape)):
         x = fft(x,axis=axis)
     return x
 
+
 def direct_idftn(x):
     x = asarray(x)
     for axis in range(len(x.shape)):
         x = ifft(x,axis=axis)
     return x
 
+
 def direct_rdft(x):
     x = asarray(x)
     n = len(x)
     w = -arange(n)*(2j*pi/n)
     r = zeros(n,dtype=double)
-    for i in range(int(n/2+1)):
+    for i in range(n//2+1):
         y = dot(exp(i*w),x)
         if i:
             r[2*i-1] = y.real
-            if 2*i<n:
+            if 2*i < n:
                 r[2*i] = y.imag
         else:
             r[0] = y.real
     return r
 
+
 def direct_irdft(x):
     x = asarray(x)
     n = len(x)
     x1 = zeros(n,dtype=cdouble)
-    for i in range(int(n/2+1)):
+    for i in range(n//2+1):
         if i:
-            if 2*i<n:
-                x1[i] = x[2*i-1] + 1j* x[2*i]
-                x1[n-i] = x[2*i-1] - 1j* x[2*i]
+            if 2*i < n:
+                x1[i] = x[2*i-1] + 1j*x[2*i]
+                x1[n-i] = x[2*i-1] - 1j*x[2*i]
             else:
                 x1[i] = x[2*i-1]
         else:
             x1[0] = x[0]
     return direct_idft(x1).real
 
+
 class _TestFFTBase(TestCase):
     def setUp(self):
         self.cdt = None
@@ -117,13 +129,13 @@ class _TestFFTBase(TestCase):
         np.random.seed(1234)
 
     def test_definition(self):
-        x = np.array([1,2,3,4+1j,1,2,3,4+2j], dtype = self.cdt)
+        x = np.array([1,2,3,4+1j,1,2,3,4+2j], dtype=self.cdt)
         y = fft(x)
         self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         y1 = direct_dft(x)
         assert_array_almost_equal(y,y1)
-        x = np.array([1,2,3,4+0j,5], dtype = self.cdt)
+        x = np.array([1,2,3,4+0j,5], dtype=self.cdt)
         assert_array_almost_equal(fft(x),direct_dft(x))
 
     def test_n_argument_real(self):
@@ -149,18 +161,24 @@ class _TestFFTBase(TestCase):
     def test_djbfft(self):
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             y = fftpack.zfft(x)
             y2 = numpy.fft.fft(x)
             assert_array_almost_equal(y,y2)
             y = fftpack.zrfft(x)
             assert_array_almost_equal(y,y2)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, fft, [])
+        assert_raises(ValueError, fft, [[1,1],[2,2]], -5)
+
+
 class TestDoubleFFT(_TestFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
 
+
 class TestSingleFFT(_TestFFTBase):
     def setUp(self):
         self.cdt = np.complex64
@@ -170,6 +188,7 @@ class TestSingleFFT(_TestFFTBase):
     def test_notice(self):
         pass
 
+
 class _TestIFFTBase(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -201,7 +220,7 @@ class _TestIFFTBase(TestCase):
     def test_djbfft(self):
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             y = fftpack.zfft(x,direction=-1)
             y2 = numpy.fft.ifft(x)
             assert_array_almost_equal(y,y2)
@@ -211,15 +230,15 @@ class _TestIFFTBase(TestCase):
     def test_random_complex(self):
         for size in [1,51,111,100,200,64,128,256,1024]:
             x = random([size]).astype(self.cdt)
-            x = random([size]).astype(self.cdt) +1j*x
+            x = random([size]).astype(self.cdt) + 1j*x
             y1 = ifft(fft(x))
             y2 = fft(ifft(x))
             self.assertTrue(y1.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
             self.assertTrue(y2.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
-            assert_array_almost_equal (y1, x)
-            assert_array_almost_equal (y2, x)
+            assert_array_almost_equal(y1, x)
+            assert_array_almost_equal(y2, x)
 
     def test_random_real(self):
         for size in [1,51,111,100,200,64,128,256,1024]:
@@ -230,8 +249,8 @@ class _TestIFFTBase(TestCase):
                     "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
             self.assertTrue(y2.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
-            assert_array_almost_equal (y1, x)
-            assert_array_almost_equal (y2, x)
+            assert_array_almost_equal(y1, x)
+            assert_array_almost_equal(y2, x)
 
     def test_size_accuracy(self):
         # Sanity check for the accuracy for prime and non-prime sized inputs
@@ -244,30 +263,37 @@ class _TestIFFTBase(TestCase):
             np.random.seed(1234)
             x = np.random.rand(size).astype(self.rdt)
             y = ifft(fft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
             y = fft(ifft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
             x = (x + 1j*np.random.rand(size)).astype(self.cdt)
             y = ifft(fft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
             y = fft(ifft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, ifft, [])
+        assert_raises(ValueError, ifft, [[1,1],[2,2]], -5)
+
+
 class TestDoubleIFFT(_TestIFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
 
+
 class TestSingleIFFT(_TestIFFTBase):
     def setUp(self):
         self.cdt = np.complex64
         self.rdt = np.float32
 
+
 class _TestRFFTBase(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -285,27 +311,34 @@ class _TestRFFTBase(TestCase):
         from numpy.fft import fft as numpy_fft
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             y2 = numpy_fft(x)
             y1 = zeros((n,),dtype=double)
             y1[0] = y2[0].real
-            y1[-1] = y2[n/2].real
-            for k in range(1, int(n/2)):
+            y1[-1] = y2[n//2].real
+            for k in range(1, n//2):
                 y1[2*k-1] = y2[k].real
                 y1[2*k] = y2[k].imag
             y = fftpack.drfft(x)
             assert_array_almost_equal(y,y1)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, rfft, [])
+        assert_raises(ValueError, rfft, [[1,1],[2,2]], -5)
+
+
 class TestRFFTDouble(_TestRFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
 
+
 class TestRFFTSingle(_TestRFFTBase):
     def setUp(self):
         self.cdt = np.complex64
         self.rdt = np.float32
 
+
 class _TestIRFFTBase(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -313,7 +346,7 @@ class _TestIRFFTBase(TestCase):
     def test_definition(self):
         x1 = [1,2,3,4,1,2,3,4]
         x1_1 = [1,2+3j,4+1j,2+3j,4,2-3j,4-1j,2-3j]
-        x2= [1,2,3,4,1,2,3,4,5]
+        x2 = [1,2,3,4,1,2,3,4,5]
         x2_1 = [1,2+3j,4+1j,2+3j,4+5j,4-5j,2-3j,4-1j,2-3j]
 
         def _test(x, xr):
@@ -331,13 +364,13 @@ class _TestIRFFTBase(TestCase):
         from numpy.fft import ifft as numpy_ifft
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             x1 = zeros((n,),dtype=cdouble)
             x1[0] = x[0]
-            for k in range(1, int(n/2)):
+            for k in range(1, n//2):
                 x1[k] = x[2*k-1]+1j*x[2*k]
                 x1[n-k] = x[2*k-1]-1j*x[2*k]
-            x1[n/2] = x[-1]
+            x1[n//2] = x[-1]
             y1 = numpy_ifft(x1)
             y = fftpack.drfft(x,direction=-1)
             assert_array_almost_equal(y,y1)
@@ -351,9 +384,9 @@ class _TestIRFFTBase(TestCase):
                     "Output dtype is %s, expected %s" % (y1.dtype, self.rdt))
             self.assertTrue(y2.dtype == self.rdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.rdt))
-            assert_array_almost_equal (y1, x, decimal=self.ndec,
+            assert_array_almost_equal(y1, x, decimal=self.ndec,
                                        err_msg="size=%d" % size)
-            assert_array_almost_equal (y2, x, decimal=self.ndec,
+            assert_array_almost_equal(y2, x, decimal=self.ndec,
                                        err_msg="size=%d" % size)
 
     def test_size_accuracy(self):
@@ -367,26 +400,34 @@ class _TestIRFFTBase(TestCase):
             np.random.seed(1234)
             x = np.random.rand(size).astype(self.rdt)
             y = irfft(rfft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
             y = rfft(irfft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, irfft, [])
+        assert_raises(ValueError, irfft, [[1,1],[2,2]], -5)
+
+
 # self.ndec is bogus; we should have a assert_array_approx_equal for number of
 # significant digits
+
 class TestIRFFTDouble(_TestIRFFTBase):
     def setUp(self):
         self.cdt = np.cdouble
         self.rdt = np.double
         self.ndec = 14
 
+
 class TestIRFFTSingle(_TestIRFFTBase):
     def setUp(self):
         self.cdt = np.complex64
         self.rdt = np.float32
         self.ndec = 5
 
+
 class Testfft2(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -397,9 +438,14 @@ class Testfft2(TestCase):
         # used
         x = numpy.ones((4,4,2))
         y = fft2(x, shape=(8,8), axes=(-3,-2))
-        y_r = numpy.fft.fftn(x, s=(8, 8), axes=(-3,  -2))
+        y_r = numpy.fft.fftn(x, s=(8, 8), axes=(-3, -2))
         assert_array_almost_equal(y, y_r)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, fft2, [[]])
+        assert_raises(ValueError, fft2, [[1,1],[2,2]], (4, -3))
+
+
 class TestFftnSingle(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -420,7 +466,7 @@ class TestFftnSingle(TestCase):
             y1 = fftn(x.real.astype(np.float32))
             y2 = fftn(x.real.astype(np.float64)).astype(np.complex64)
 
-            self.failUnless(y1.dtype == np.complex64)
+            self.assertTrue(y1.dtype == np.complex64)
             assert_array_almost_equal_nulp(y1, y2, 2000)
 
         for size in LARGE_COMPOSITE_SIZES + LARGE_PRIME_SIZES:
@@ -429,9 +475,10 @@ class TestFftnSingle(TestCase):
             y1 = fftn(x.real.astype(np.float32))
             y2 = fftn(x.real.astype(np.float64)).astype(np.complex64)
 
-            self.failUnless(y1.dtype == np.complex64)
+            self.assertTrue(y1.dtype == np.complex64)
             assert_array_almost_equal_nulp(y1, y2, 2000)
 
+
 class TestFftn(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -446,7 +493,7 @@ class TestFftn(TestCase):
         assert_array_almost_equal(fftn(x),direct_dftn(x))
 
     def test_axes_argument(self):
-        #plane == ji_plane, x== kji_space
+        # plane == ji_plane, x== kji_space
         plane1 = [[1,2,3],[4,5,6],[7,8,9]]
         plane2 = [[10,11,12],[13,14,15],[16,17,18]]
         plane3 = [[19,20,21],[22,23,24],[25,26,27]]
@@ -472,76 +519,76 @@ class TestFftn(TestCase):
         kij_space = [ij_plane1,ij_plane2,ij_plane3]
         x = array([plane1,plane2,plane3])
 
-        assert_array_almost_equal(fftn(x),fftn(x,axes=(-3,-2,-1))) # kji_space
+        assert_array_almost_equal(fftn(x),fftn(x,axes=(-3,-2,-1)))  # kji_space
         assert_array_almost_equal(fftn(x),fftn(x,axes=(0,1,2)))
-        y = fftn(x,axes=(2,1,0)) # ijk_space
+        y = fftn(x,axes=(2,1,0))  # ijk_space
         assert_array_almost_equal(swapaxes(y,-1,-3),fftn(ijk_space))
-        y = fftn(x,axes=(2,0,1)) # ikj_space
+        y = fftn(x,axes=(2,0,1))  # ikj_space
         assert_array_almost_equal(swapaxes(swapaxes(y,-1,-3),
-                                                   -1,-2)
-                                  ,fftn(ikj_space))
-        y = fftn(x,axes=(1,2,0)) # jik_space
+                                                   -1,-2),
+                                  fftn(ikj_space))
+        y = fftn(x,axes=(1,2,0))  # jik_space
         assert_array_almost_equal(swapaxes(swapaxes(y,-1,-3),
-                                                   -3,-2)
-                                  ,fftn(jik_space))
-        y = fftn(x,axes=(1,0,2)) # jki_space
+                                                   -3,-2),
+                                  fftn(jik_space))
+        y = fftn(x,axes=(1,0,2))  # jki_space
         assert_array_almost_equal(swapaxes(y,-2,-3),fftn(jki_space))
-        y = fftn(x,axes=(0,2,1)) # kij_space
+        y = fftn(x,axes=(0,2,1))  # kij_space
         assert_array_almost_equal(swapaxes(y,-2,-1),
                                   fftn(kij_space))
 
-        y = fftn(x,axes=(-2,-1)) # ji_plane
+        y = fftn(x,axes=(-2,-1))  # ji_plane
         assert_array_almost_equal(fftn(plane1),y[0])
         assert_array_almost_equal(fftn(plane2),y[1])
         assert_array_almost_equal(fftn(plane3),y[2])
-        y = fftn(x,axes=(1,2)) # ji_plane
+        y = fftn(x,axes=(1,2))  # ji_plane
         assert_array_almost_equal(fftn(plane1),y[0])
         assert_array_almost_equal(fftn(plane2),y[1])
         assert_array_almost_equal(fftn(plane3),y[2])
-        y = fftn(x,axes=(-3,-2)) # kj_plane
+        y = fftn(x,axes=(-3,-2))  # kj_plane
         assert_array_almost_equal(fftn(x[:,:,0]),y[:,:,0])
         assert_array_almost_equal(fftn(x[:,:,1]),y[:,:,1])
         assert_array_almost_equal(fftn(x[:,:,2]),y[:,:,2])
-        y = fftn(x,axes=(-3,-1)) # ki_plane
+        y = fftn(x,axes=(-3,-1))  # ki_plane
         assert_array_almost_equal(fftn(x[:,0,:]),y[:,0,:])
         assert_array_almost_equal(fftn(x[:,1,:]),y[:,1,:])
         assert_array_almost_equal(fftn(x[:,2,:]),y[:,2,:])
-        y = fftn(x,axes=(-1,-2)) # ij_plane
+        y = fftn(x,axes=(-1,-2))  # ij_plane
         assert_array_almost_equal(fftn(ij_plane1),swapaxes(y[0],-2,-1))
         assert_array_almost_equal(fftn(ij_plane2),swapaxes(y[1],-2,-1))
         assert_array_almost_equal(fftn(ij_plane3),swapaxes(y[2],-2,-1))
-        y = fftn(x,axes=(-1,-3)) # ik_plane
+        y = fftn(x,axes=(-1,-3))  # ik_plane
         assert_array_almost_equal(fftn(ik_plane1),swapaxes(y[:,0,:],-1,-2))
         assert_array_almost_equal(fftn(ik_plane2),swapaxes(y[:,1,:],-1,-2))
         assert_array_almost_equal(fftn(ik_plane3),swapaxes(y[:,2,:],-1,-2))
-        y = fftn(x,axes=(-2,-3)) # jk_plane
+        y = fftn(x,axes=(-2,-3))  # jk_plane
         assert_array_almost_equal(fftn(jk_plane1),swapaxes(y[:,:,0],-1,-2))
         assert_array_almost_equal(fftn(jk_plane2),swapaxes(y[:,:,1],-1,-2))
         assert_array_almost_equal(fftn(jk_plane3),swapaxes(y[:,:,2],-1,-2))
 
-        y = fftn(x,axes=(-1,)) # i_line
+        y = fftn(x,axes=(-1,))  # i_line
         for i in range(3):
             for j in range(3):
                 assert_array_almost_equal(fft(x[i,j,:]),y[i,j,:])
-        y = fftn(x,axes=(-2,)) # j_line
+        y = fftn(x,axes=(-2,))  # j_line
         for i in range(3):
             for j in range(3):
                 assert_array_almost_equal(fft(x[i,:,j]),y[i,:,j])
-        y = fftn(x,axes=(0,)) # k_line
+        y = fftn(x,axes=(0,))  # k_line
         for i in range(3):
             for j in range(3):
                 assert_array_almost_equal(fft(x[:,i,j]),y[:,i,j])
 
-        y = fftn(x,axes=()) # point
+        y = fftn(x,axes=())  # point
         assert_array_almost_equal(y,x)
 
     def test_shape_argument(self):
         small_x = [[1,2,3],[4,5,6]]
         large_x1 = [[1,2,3,0],[4,5,6,0],[0,0,0,0],[0,0,0,0]]
         y = fftn(small_x,shape=(4,4))
-        assert_array_almost_equal (y,fftn(large_x1))
+        assert_array_almost_equal(y,fftn(large_x1))
         y = fftn(small_x,shape=(3,4))
-        assert_array_almost_equal (y,fftn(large_x1[:-1]))
+        assert_array_almost_equal(y,fftn(large_x1[:-1]))
 
     def test_shape_axes_argument(self):
         small_x = [[1,2,3],[4,5,6],[7,8,9]]
@@ -550,16 +597,16 @@ class TestFftn(TestCase):
                                   [7,8,9,0],
                                   [0,0,0,0]])
         # Disable tests with shape and axes of different lengths
-        #y = fftn(small_x,shape=(4,4),axes=(-1,))
-        #for i in range(4):
+        # y = fftn(small_x,shape=(4,4),axes=(-1,))
+        # for i in range(4):
         #    assert_array_almost_equal (y[i],fft(large_x1[i]))
-        #y = fftn(small_x,shape=(4,4),axes=(-2,))
-        #for i in range(4):
+        # y = fftn(small_x,shape=(4,4),axes=(-2,))
+        # for i in range(4):
         #    assert_array_almost_equal (y[:,i],fft(large_x1[:,i]))
         y = fftn(small_x,shape=(4,4),axes=(-2,-1))
-        assert_array_almost_equal (y,fftn(large_x1))
+        assert_array_almost_equal(y,fftn(large_x1))
         y = fftn(small_x,shape=(4,4),axes=(-1,-2))
-        assert_array_almost_equal (y,swapaxes(\
+        assert_array_almost_equal(y,swapaxes(
             fftn(swapaxes(large_x1,-1,-2)),-1,-2))
 
     def test_shape_axes_argument2(self):
@@ -583,6 +630,10 @@ class TestFftn(TestCase):
         x = zeros((4, 4, 2))
         assert_raises(ValueError, fftn, x, shape=(8, 8, 2, 1))
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, fftn, [[]])
+        assert_raises(ValueError, fftn, [[1,1],[2,2]], (4, -3))
+
 
 class _TestIfftn(TestCase):
     dtype = None
@@ -607,16 +658,23 @@ class _TestIfftn(TestCase):
             assert_array_almost_equal_nulp(ifftn(fftn(x)),x,self.maxnlp)
             assert_array_almost_equal_nulp(fftn(ifftn(x)),x,self.maxnlp)
 
+    def test_invalid_sizes(self):
+        assert_raises(ValueError, ifftn, [[]])
+        assert_raises(ValueError, ifftn, [[1,1],[2,2]], (4, -3))
+
+
 class TestIfftnDouble(_TestIfftn):
     dtype = np.float64
     cdtype = np.complex128
     maxnlp = 2000
 
+
 class TestIfftnSingle(_TestIfftn):
     dtype = np.float32
     cdtype = np.complex64
     maxnlp = 3500
 
+
 class TestLongDoubleFailure(TestCase):
     def setUp(self):
         np.random.seed(1234)
@@ -632,7 +690,7 @@ class TestLongDoubleFailure(TestCase):
         for f in [fft, ifft]:
             try:
                 f(x)
-                raise AssertionError("Type %r not supported but does not fail" % \
+                raise AssertionError("Type %r not supported but does not fail" %
                                      np.longcomplex)
             except ValueError:
                 pass
@@ -647,24 +705,26 @@ class TestLongDoubleFailure(TestCase):
         for f in [fft, ifft]:
             try:
                 f(x)
-                raise AssertionError("Type %r not supported but does not fail" % \
+                raise AssertionError("Type %r not supported but does not fail" %
                                      np.longcomplex)
             except ValueError:
                 pass
 
 
-
 class FakeArray(object):
     def __init__(self, data):
         self._data = data
         self.__array_interface__ = data.__array_interface__
 
+
 class FakeArray2(object):
     def __init__(self, data):
         self._data = data
+
     def __array__(self):
         return self._data
 
+
 class TestOverwrite(object):
     """
     Check input overwrite behavior of the FFT functions
@@ -682,9 +742,6 @@ class TestOverwrite(object):
                 routine.__name__, x.dtype, x.shape, fftsize, axis, overwrite_x)
             if not should_overwrite:
                 assert_equal(x2, x, err_msg="spurious overwrite in %s" % sig)
-            else:
-                if (x2 == x).all():
-                    raise AssertionError("no overwrite in %s" % sig)
 
     def _check_1d(self, routine, dtype, shape, axis, overwritable_dtypes):
         np.random.seed(1234)
