@@ -26,10 +26,11 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
+from __future__ import division, print_function, absolute_import
 
 import struct
 import numpy as np
-from numpy.compat import asbytes, asstr
+from numpy.compat import asstr
 import tempfile
 import zlib
 import warnings
@@ -183,41 +184,41 @@ def _read_string_data(f):
 
 def _read_data(f, dtype):
     '''Read a variable with a specified data type'''
-    if dtype==1:
+    if dtype == 1:
         if _read_int32(f) != 1:
             raise Exception("Error occurred while reading byte variable")
         return _read_byte(f)
-    elif dtype==2:
+    elif dtype == 2:
         return _read_int16(f)
-    elif dtype==3:
+    elif dtype == 3:
         return _read_int32(f)
-    elif dtype==4:
+    elif dtype == 4:
         return _read_float32(f)
-    elif dtype==5:
+    elif dtype == 5:
         return _read_float64(f)
-    elif dtype==6:
+    elif dtype == 6:
         real = _read_float32(f)
         imag = _read_float32(f)
         return np.complex64(real + imag * 1j)
-    elif dtype==7:
+    elif dtype == 7:
         return _read_string_data(f)
-    elif dtype==8:
+    elif dtype == 8:
         raise Exception("Should not be here - please report this")
-    elif dtype==9:
+    elif dtype == 9:
         real = _read_float64(f)
         imag = _read_float64(f)
         return np.complex128(real + imag * 1j)
-    elif dtype==10:
+    elif dtype == 10:
         return Pointer(_read_int32(f))
-    elif dtype==11:
+    elif dtype == 11:
         return ObjectPointer(_read_int32(f))
-    elif dtype==12:
+    elif dtype == 12:
         return _read_uint16(f)
-    elif dtype==13:
+    elif dtype == 13:
         return _read_uint32(f)
-    elif dtype==14:
+    elif dtype == 14:
         return _read_int64(f)
-    elif dtype==15:
+    elif dtype == 15:
         return _read_uint64(f)
     else:
         raise Exception("Unknown IDL type: %i - please report this" % dtype)
@@ -251,11 +252,11 @@ def _read_structure(f, array_desc, struct_desc):
         for col in columns:
             dtype = col['typecode']
             if col['structure']:
-                structure[col['name']][i] = _read_structure(f, \
-                                      struct_desc['arrtable'][col['name']], \
+                structure[col['name']][i] = _read_structure(f,
+                                      struct_desc['arrtable'][col['name']],
                                       struct_desc['structtable'][col['name']])
             elif col['array']:
-                structure[col['name']][i] = _read_array(f, dtype, \
+                structure[col['name']][i] = _read_array(f, dtype,
                                       struct_desc['arrtable'][col['name']])
             else:
                 structure[col['name']][i] = _read_data(f, dtype)
@@ -284,14 +285,14 @@ def _read_array(f, typecode, array_desc):
                 raise Exception("Error occurred while reading byte array")
 
         # Read bytes as numpy array
-        array = np.fromstring(f.read(array_desc['nbytes']), \
+        array = np.fromstring(f.read(array_desc['nbytes']),
                                 dtype=DTYPE_DICT[typecode])
 
     elif typecode in [2, 12]:
 
         # These are 2 byte types, need to skip every two as they are not packed
 
-        array = np.fromstring(f.read(array_desc['nbytes']*2), \
+        array = np.fromstring(f.read(array_desc['nbytes']*2),
                                 dtype=DTYPE_DICT[typecode])[1::2]
 
     else:
@@ -350,10 +351,10 @@ def _read_record(f):
             raise Exception("VARSTART is not 7")
 
         if rectypedesc['structure']:
-            record['data'] = _read_structure(f, rectypedesc['array_desc'], \
+            record['data'] = _read_structure(f, rectypedesc['array_desc'],
                                           rectypedesc['struct_desc'])
         elif rectypedesc['array']:
-            record['data'] = _read_array(f, rectypedesc['typecode'], \
+            record['data'] = _read_array(f, rectypedesc['typecode'],
                                       rectypedesc['array_desc'])
         else:
             dtype = rectypedesc['typecode']
@@ -412,7 +413,7 @@ def _read_record(f):
 
     else:
 
-        raise Exception("record['rectype']=%s not implemented" % \
+        raise Exception("record['rectype']=%s not implemented" %
                                                             record['rectype'])
 
     f.seek(nextrec)
@@ -668,7 +669,7 @@ class AttrDict(dict):
 
 def readsav(file_name, idict=None, python_dict=False,
             uncompressed_file_name=None, verbose=False):
-    '''
+    """
     Read an IDL .sav file
 
     Parameters
@@ -677,7 +678,7 @@ def readsav(file_name, idict=None, python_dict=False,
         Name of the IDL save file.
     idict : dict, optional
         Dictionary in which to insert .sav file variables
-    python_dict: bool, optional
+    python_dict : bool, optional
         By default, the object return is not a Python dictionary, but a
         case-insensitive dictionary with item, attribute, and call access
         to variables. To get a standard Python dictionary, set this option
@@ -694,7 +695,7 @@ def readsav(file_name, idict=None, python_dict=False,
         the records read, and available variables.
 
     Returns
-    ----------
+    -------
     idl_dict : AttrDict or dict
         If `python_dict` is set to False (default), this function returns a
         case-insensitive dictionary with item, attribute, and call access
@@ -702,7 +703,8 @@ def readsav(file_name, idict=None, python_dict=False,
         returns a Python dictionary with all variable names in lowercase.
         If `idict` was specified, then variables are written to the
         dictionary specified, and the updated dictionary is returned.
-    '''
+
+    """
 
     # Initialize record and variable holders
     records = []
@@ -716,20 +718,20 @@ def readsav(file_name, idict=None, python_dict=False,
 
     # Read the signature, which should be 'SR'
     signature = _read_bytes(f, 2)
-    if signature != asbytes('SR'):
+    if signature != b'SR':
         raise Exception("Invalid SIGNATURE: %s" % signature)
 
     # Next, the record format, which is '\x00\x04' for normal .sav
     # files, and '\x00\x06' for compressed .sav files.
     recfmt = _read_bytes(f, 2)
 
-    if recfmt == asbytes('\x00\x04'):
+    if recfmt == b'\x00\x04':
         pass
 
-    elif recfmt == asbytes('\x00\x06'):
+    elif recfmt == b'\x00\x06':
 
         if verbose:
-            print "IDL Save file is compressed"
+            print("IDL Save file is compressed")
 
         if uncompressed_file_name:
             fout = open(uncompressed_file_name, 'w+b')
@@ -737,10 +739,10 @@ def readsav(file_name, idict=None, python_dict=False,
             fout = tempfile.NamedTemporaryFile(suffix='.sav')
 
         if verbose:
-            print " -> expanding to %s" % fout.name
+            print(" -> expanding to %s" % fout.name)
 
         # Write header
-        fout.write(asbytes('SR\x00\x04'))
+        fout.write(b'SR\x00\x04')
 
         # Cycle through records
         while True:
@@ -818,48 +820,48 @@ def readsav(file_name, idict=None, python_dict=False,
         # Print out timestamp info about the file
         for record in records:
             if record['rectype'] == "TIMESTAMP":
-                print "-"*50
-                print "Date: %s" % record['date']
-                print "User: %s" % record['user']
-                print "Host: %s" % record['host']
+                print("-"*50)
+                print("Date: %s" % record['date'])
+                print("User: %s" % record['user'])
+                print("Host: %s" % record['host'])
                 break
 
         # Print out version info about the file
         for record in records:
             if record['rectype'] == "VERSION":
-                print "-"*50
-                print "Format: %s" % record['format']
-                print "Architecture: %s" % record['arch']
-                print "Operating System: %s" % record['os']
-                print "IDL Version: %s" % record['release']
+                print("-"*50)
+                print("Format: %s" % record['format'])
+                print("Architecture: %s" % record['arch'])
+                print("Operating System: %s" % record['os'])
+                print("IDL Version: %s" % record['release'])
                 break
 
         # Print out identification info about the file
         for record in records:
             if record['rectype'] == "IDENTIFICATON":
-                print "-"*50
-                print "Author: %s" % record['author']
-                print "Title: %s" % record['title']
-                print "ID Code: %s" % record['idcode']
+                print("-"*50)
+                print("Author: %s" % record['author'])
+                print("Title: %s" % record['title'])
+                print("ID Code: %s" % record['idcode'])
                 break
 
-        print "-"*50
-        print "Successfully read %i records of which:" % \
-                                            (len(records))
+        print("-"*50)
+        print("Successfully read %i records of which:" %
+                                            (len(records)))
 
         # Create convenience list of record types
         rectypes = [r['rectype'] for r in records]
 
         for rt in set(rectypes):
             if rt != 'END_MARKER':
-                print " - %i are of type %s" % (rectypes.count(rt), rt)
-        print "-"*50
+                print(" - %i are of type %s" % (rectypes.count(rt), rt))
+        print("-"*50)
 
         if 'VARIABLE' in rectypes:
-            print "Available variables:"
+            print("Available variables:")
             for var in variables:
-                print " - %s [%s]" % (var, type(variables[var]))
-            print "-"*50
+                print(" - %s [%s]" % (var, type(variables[var])))
+            print("-"*50)
 
     if idict:
         for var in variables:
